# Author: Darrell Christmas
# Last Updated: 10/01/08
# Enemy rat class

from pandac.PandaModules import *
from direct.interval.MetaInterval import Sequence
from direct.actor.Actor import Actor
from direct.showbase.DirectObject import DirectObject
from State.RatStates import PatrolState
from State.RatStates import PursuitState
from State.StateMachine import StateMachine
from direct.task.Task import Task
import math


MODEL_PATH = "../models/rat/"
class Rat(DirectObject):
    def __init__(self, parentObject, playerEntity, id, position, heading, standing):
        self.parent = parentObject
        self.player = playerEntity
        self.id = id
        self.pursuitDist = 15
        self.isMoving = False
        self.isStanding = standing
        
        #self.actorRoot = render.attachNewNode("ratRoot")
        self.actor = Actor(MODEL_PATH+"Rat_Run_Model", {"walk": MODEL_PATH+"Rat_Run_Animation", "run":MODEL_PATH+"Rat_Run_Animation", "idle": MODEL_PATH+"Rat_Idle_Animation", "death":MODEL_PATH+"Rat_Death_Animation"})
        self.actor.setPlayRate(0.4, "walk")
        self.actor.reparentTo(render)
        self.actor.setScale(.5)
        
         #
        self.pursuitVelocity = 0.08
        self.patrolRotation = 0
        self.patrolVelocity = 0.07
        self.patrolBounds = 10
        self.patrolPos = Point3(0,0,0)
        self.patrolHeading = 0
        self.setPatrolPos(position)
        self.setPatrolHeading (heading)
        #
        
        
        
        
       
        #state variables
        self.isPlayerSeen = False
        self.isPlayerInRange = False
        self.isWallInVision = False
        self.sneakSeen = False
        
        self.onPatrolRoute = True
        self.isPatrolling = False
        
        
        
        
        self.stateMachine = StateMachine(self)
        patrolState = PatrolState()
        self.stateMachine.setCurrentState(patrolState)
        
        #self.stateMachine.setGlobalState(PatrolState())
        self.fsmUpdate = taskMgr.add(self.fsmUpdateTask, "fsmUpdateTask")
        self.setUpCollision()
        
     
        return None
    

    
    #input in vec3 format
    def setPatrolPos(self, pos):
        self.patrolPos = pos
        self.actor.setPos (pos)
        return None
    
    def setPatrolHeading(self, heading):
        self.patrolHeading = heading
        self.actor.setH (heading)
        return None
    def setStandingState(self, standing):
        self.isStanding = standing
        
        return None    

    def setUpCollision(self): 
        #from collider for wall vision
        self.cSolid = self.actor.attachNewNode(CollisionNode('ratVision'))
        self.cSolid.node().addSolid(CollisionSegment(0, 0, 2, 0, -20, 2))
        self.cSolid.setTag( "id", str(self.id) )
        self.cSolid.node().setFromCollideMask(BitMask32.bit(2))
        self.cSolid.node().setIntoCollideMask(BitMask32.allOff())
        #self.cSolid.show()
        #add collider to traverser queue
        self.parent.colMan.addColliderEvent(self.cSolid)
        
        #other collision
        self.fromObject = self.actor.attachNewNode(CollisionNode('ratMain'))
        self.fromObject.setTag( "id", str(self.id) )
        self.fromObject.node().addSolid(CollisionSphere(0,0,2,1))
        self.fromObject.node().addSolid(CollisionSegment(0, 0, 0, 0, 0, -3))
        #self.actor.node().setIntoCollideMask(BitMask32.allOff())
        #self.fromObject.node().setIntoCollideMask(BitMask32.bit(3))
        #self.fromObject.node().setFromCollideMask(BitMask32.bit(3))
        #self.fromObject.show()
        #add squirrel collision to traverser
        self.parent.colMan.addColliderEvent(self.fromObject)
        
        return None
        
    #
    def getPlayerDist(self):    
        player_disty = abs(self.player.getY() - self.actor.getY())
        player_distx = abs(self.player.getX() - self.actor.getX())
        self.player_dist = math.sqrt((player_distx ** 2) + (player_disty ** 2))
        #print self.player_dist
        if (self.player_dist <= self.pursuitDist ):
            self.isPlayerInRange = True
        else :
            self.isPlayerInRange = False
            
        zpos = abs(self.player.getZ() - self.actor.getZ())
           
        if(zpos >= 1):
            self.isPlayerInRange = False
            
        return self.player_dist
    
    #
    def getWallDist(self, wallVec):
        wall_disty = abs(wallVec.getY())
        wall_distx = abs(wallVec.getX())
        wall_dist = math.sqrt((wall_distx ** 2) + (wall_disty ** 2))
        if(self.player_dist > wall_dist ):
            self.isWallInVision = True
        else:
            self.isWallInVision = False
        
        return wall_dist   
    
    #  
    def getPlayerSneak(self):
        
        is_sneak = self.player.isSneaking
        dist = self.pursuitDist/4
        if (is_sneak == False or dist > self.player_dist):
            self.sneakSeen = True
        else:
            self.sneakSeen = False
        return self.sneakSeen
       
    def getPlayerSeen(self):
        player_vector = self.player.getPos() - self.actor.getPos()
        self_vector = - self.actor.getNetTransform().getMat().getRow3(1)
        player_vector.normalize()
        self_vector.normalize()
        angle = self_vector.dot(player_vector)
   
        if (angle >= 0.80 and angle <= 1 or self.sneakSeen == True ):
            #print angle
            self.isPlayerSeen = True
        else:
            self.isPlayerSeen = False
        
            
        return self.isPlayerSeen
    #        
    def fsmUpdateTask(self,task):
        self.getPlayerDist() 
        self.getPlayerSeen()
        self.getPlayerSneak()
        self.stateMachine.update()
        return Task.cont
    
    #  
    def getIsPlayerSeen(self):
        return self.isPlayerSeen
    
    #    
    def getisPlayerInRange(self):
        return self.isPlayerInRange
    
    #
    def doPursuit(self):
        self.isOnPatrolRoute = False
        forward = -self.actor.getNetTransform().getMat().getRow3(1)
        forward.setZ(0)
        forward.normalize()
        player_vector = self.player.getPos() - self.actor.getPos()
        player_vector.setZ(0)
        player_vector.normalize()
        angle = self.findAngleBetweenVectors(forward, player_vector)
        crossProd = forward.cross(player_vector)
        
        if (crossProd.getZ() > 0):
            self.actor.setH(self.actor.getH() + 2)
        elif (crossProd.getZ() < 0):
            self.actor.setH(self.actor.getH() - 2)
            
        self.actor.setPos(self.actor.getPos() + player_vector*(self.pursuitVelocity * 1))
        if self.isMoving is False:
            self.actor.loop("run")
            self.isMoving = True
        return None
    
    #
    def doPatrol(self):
        if(self.isStanding == False):
        
            forward = -self.actor.getNetTransform().getMat().getRow3(1)
            forward.setZ(0)
            forward.normalize()
            
            
            if(self.patrolHeading == 0 or self.patrolHeading == 180):
                dispPos = self.patrolPos.getY() + self.patrolBounds
                dispNeg = self.patrolPos.getY() - self.patrolBounds
            
                   
                if(self.actor.getY() < dispPos and self.actor.getY() > dispNeg):
                    self.actor.setPos(self.actor.getPos() + forward*(self.patrolVelocity*1))
                elif (self.actor.getY() <= dispPos):
                    if(self.patrolRotation < 20):
                        self.actor.setH(self.actor.getH() + 9)
                        self.patrolRotation = self.patrolRotation + 1
                    else:
                        self.patrolRotation = 0
                        self.actor.setPos(self.actor.getPos() + forward*(self.patrolVelocity*1))
                elif (self.actor.getY() >= dispNeg):
                    if(self.patrolRotation < 20):
                        self.actor.setH(self.actor.getH() - 9)
                        self.patrolRotation = self.patrolRotation + 1
                    else:
                        self.patrolRotation = 0
                        self.actor.setPos(self.actor.getPos() + forward*(self.patrolVelocity*1))
             
             
            elif (self.patrolHeading == 90 or self.patrolHeading == 270):
                dispPos = self.patrolPos.getX() + self.patrolBounds
                dispNeg = self.patrolPos.getX() - self.patrolBounds 
                
                if(self.actor.getX() < dispPos and self.actor.getX() > dispNeg):
                    self.actor.setPos(self.actor.getPos() + forward*(self.patrolVelocity*1))
                elif (self.actor.getX() <= dispPos):
                    if(self.patrolRotation < 20):
                        self.actor.setH(self.actor.getH() + 9)
                        self.patrolRotation = self.patrolRotation + 1
                    else:
                        self.patrolRotation = 0
                        self.actor.setPos(self.actor.getPos() + forward*(self.patrolVelocity*1))
                elif (self.actor.getX() >= dispNeg):
                    if(self.patrolRotation < 20):
                        self.actor.setH(self.actor.getH() - 9)
                        self.patrolRotation = self.patrolRotation + 1
                    else:
                        self.patrolRotation = 0
                        self.actor.setPos(self.actor.getPos() + forward*(self.patrolVelocity*1))
             
             
                
                
            self.isPatrolling = True
            self.isOnPatrolRoute = True
            if self.isMoving is False:
                self.actor.loop("walk")
                self.isMoving = True
        else:
            self.isPatrolling = True
            self.isOnPatrolRoute = True
            if self.isMoving is False:
                self.actor.loop("idle")
                self.isMoving = True
                
            
        
        #print self.actor.getAnimControl("walk").isPlaying()
        return None
    
    def isOnPatrolRoute(self):
        return self.onPatrolRoute
    
    def setOnPatrolRoute(self, value):
        self.onPatrolRoute = value
        return None
    
    def gotoPatrolRoute(self):
        self.actor.setPos(self.patrolPos)
        self.actor.setH(self.patrolHeading)
        return None
    
    def doStopPatrol(self):
        self.actor.stop()
        self.isMoving = False
        self.isPatrolling = False
        return None
        
    def doStopPursuit(self):
        self.actor.stop()
        self.isMoving = False
        return None
        
    # Function for finding the angle between two vectors.
    # Returns the angle in degrees!
    # Provided by Ryan Morse
    def findAngleBetweenVectors(self, vec1, vec2):
        vec1.normalize()
        vec2.normalize()
        theta = vec1.dot(vec2)
        if ( theta > 1 ):
            theta = 1
        elif ( theta < -1 ):
            theta = -1
        return math.acos(theta) * (180/math.pi)
    