# Author: Darrell Christmas
# Last Updated: 10/01/08

from pandac.PandaModules import *
from direct.actor.Actor import Actor
from direct.showbase.DirectObject import DirectObject
from direct.task.Task import Task
from direct.interval.ProjectileInterval import ProjectileInterval
MODEL_PATH = "../models/Squirrel/"

class Squirrel(DirectObject):
    def __init__(self, parentObject):
        self.parent = parentObject
        self.squirrelRoot = render.attachNewNode("squirrelRoot")
        self.squirrel = Actor(MODEL_PATH+"Squirrel_Idle_Model", {"run": MODEL_PATH+"squirrel-run", 
                                                                 "idle": MODEL_PATH+"Squirrel_Idle_Animation", 
                                                                 "sneak": MODEL_PATH+"Squirrel_Sneak_Animation", 
                                                                 "sneakBackwards": MODEL_PATH+"Squirrel_Sneak_Backwards_Animation",
                                                                 "throw": MODEL_PATH+"Squirrel_Throw_Animation"})
        self.squirrel.reparentTo(render)
        self.squirrel.setScale(0.3)
        self.squirrel.setPos(0,-10,64.5)
        self.squirrel.setH(275)
        self.isMoving = False
        self.running = False
        self.isSneaking = False
        #init camera pos
        base.camera.reparentTo(self.squirrel)
        base.camera.setPos(0,35,12)
        base.camera.setHpr(180,-10,0)
        #set up collision
        self.setupCollision()
        
        return None
        
    #set up the collision for player character
    def setupCollision(self):
        self.fromObject = self.squirrel.attachNewNode(CollisionNode('squirrel'))
        self.fromObject.node().addSolid(CollisionSphere(0,0,3,2))
        self.fromObject.node().addSolid(CollisionSegment(0, 0, 1, 0, 0, -2))
        self.squirrel.node().setIntoCollideMask(BitMask32.allOff())
        #self.fromObject.node().setIntoCollideMask(BitMask32.allOff())
        #self.fromObject.node().setFromCollideMask(BitMask32.bit(1))
        #self.fromObject.show()
        #add squirrel collision to traverser
        #self.parent.colMan.addCollider(self.fromObject)
        self.parent.colMan.addColliderEvent(self.fromObject)
        return None
    
    #move the character forward
    def moveForward(self, elapsed, sneak):
        if (sneak == True):
            self.isSneaking = True
            speed = 3
        else:
            self.isSneaking = False
            speed = 10
        camright = base.camera.getNetTransform().getMat().getRow3(0)
        camright.normalize()
        backward = self.squirrel.getNetTransform().getMat().getRow3(1)
        backward.setZ(0)
        backward.normalize()
        self.squirrel.setPos(self.squirrel.getPos() - backward*(elapsed*speed))
       
        return None
        
    #move the character backward
    def moveBackward(self, elapsed, sneak):
        if (sneak == True):
            self.isSneaking = True
            speed = 3
        else:
            self.isSneaking = False
            speed = 10
        camright = base.camera.getNetTransform().getMat().getRow3(0)
        camright.normalize()
        backward = self.squirrel.getNetTransform().getMat().getRow3(1)
        backward.setZ(0)
        backward.normalize()
        self.squirrel.setPos(self.squirrel.getPos() + backward*(elapsed*speed))
       
        return None
    
    #turn the character left
    def turnLeft(self, elapsed, sneak):
        if (sneak == True):
            self.isSneaking = True
            speed = 100
        else:
            self.isSneaking = False
            speed = 300
        camright = base.camera.getNetTransform().getMat().getRow3(0)
        camright.normalize()
        base.camera.setPos(base.camera.getPos() + camright*(elapsed*2))
        self.squirrel.setH(self.squirrel.getH() + elapsed*speed)
      
        return None
    #turn the character right    
    def turnRight(self, elapsed, sneak):
        if (sneak == True):
            self.isSneaking = True
            speed = 100
        else:
            self.isSneaking = False
            speed = 300
        camright = base.camera.getNetTransform().getMat().getRow3(0)
        camright.normalize()
        base.camera.setPos(base.camera.getPos() - camright*(elapsed*2))
        self.squirrel.setH(self.squirrel.getH() - elapsed*speed)
       
        return None
    
    #sneaking motion
    def sneak(self, elapsed):
        
        return None
    def getPos(self):
        return self.squirrel.getPos()
    def getX(self):
        return self.squirrel.getX()
    def getY(self):
        return self.squirrel.getY()
    def getZ(self):
        return self.squirrel.getZ()
    #thow an object at a target
    
    
    
    #the idle anim
    #TODO: extend the timeout time somehow    
    def idleAnim(self, task):
        if (self.isMoving == False):
            self.squirrel.loop("idle")  
        return Task.done   
    
    def dropToTunnel(self):
        #ProjectileInterval(self.missile, duration=0.4, endPos=self.target.getPos()).start()
        ProjectileInterval(self.squirrel, duration=2, endPos = Point3(self.squirrel.getX()+5,self.squirrel.getY(),45),  name = "dropToTunnel").start()
       
        return None
             

             
        