# Author: Darrell Christmas
# Last Updated: 10/01/08

from direct.task.Task import Task
from direct.showbase.DirectObject import DirectObject
from pandac.PandaModules import CollisionTraverser,CollisionNode,CollisionSphere,CollisionHandlerEvent
from pandac.PandaModules import CollisionHandlerQueue,CollisionRay,CollisionHandlerPusher

class CollisionManager(DirectObject):
    def __init__(self, parentObject):
        self.parent = parentObject
        self.prevtime = 0
        self.roofTrigger = False
        self.traverser = CollisionTraverser()
        base.cTrav = self.traverser
        self.queue = CollisionHandlerQueue()
        
        self.traverser.traverse(render)
        #self.colLoop = taskMgr.add(self.colTask, "colTask")
        #self.traverser.showCollisions(render)
        
        #set up secondary collision handler
        self.colHandler = CollisionHandlerEvent()
        self.colHandler.addInPattern('%fn-into-%in')
        self.colHandler.addAgainPattern('%fn-again-%in') 
        self.colHandler.addOutPattern('%fn-out-%in') 
        
        
        
        
        
        self.accept('ratVision-into-wall_collide', self.enemyWallVision)
        self.accept('ratVision-again-wall_collide', self.enemyWallVision)
        self.accept('ratVision-out-wall_collide', self.enemyWallVisionOut)
        self.accept('ratVision-into-box_collide', self.enemyWallVision)
        self.accept('ratVision-again-box_collide', self.enemyWallVision)
        self.accept('ratVision-out-box_collide', self.enemyWallVisionOut)
        
        
        self.accept('squirrel-into-roofTrigger', self.roofEventHandler)
        self.accept('squirrel-into-wall_collide', self.wallCollideHandler)
        self.accept('squirrel-again-wall_collide', self.wallCollideHandler)
        self.accept('squirrel-into-floor_collide', self.groundCollideHandler)
        self.accept('squirrel-again-floor_collide', self.groundCollideHandler)
        self.accept('squirrel-into-box_collide', self.wallCollideHandler)
        self.accept('squirrel-again-box_collide', self.wallCollideHandler)
        
        self.accept('ratMain-into-wall_collide', self.wallCollideHandler)
        self.accept('ratMain-again-wall_collide', self.wallCollideHandler)
        self.accept('ratMain-into-floor_collide', self.groundCollideHandler)
        self.accept('ratMain-again-floor_collide', self.groundCollideHandler)
        self.accept('ratMain-into-box_collide', self.wallCollideHandler)
        self.accept('ratMain-again-box_collide', self.wallCollideHandler)
        
        self.accept('squirrel-into-roofTrigger', self.roofEventHandler)
        self.accept('squirrel-into-ventTrigger1', self.ventConvoHandler)
        self.accept('squirrel-into-ventTrigger2', self.ventConvoHandler)
        self.accept('squirrel-into-ventTrigger3', self.ventConvoHandler)
        self.accept('squirrel-into-ventExitTrigger', self.ventExitHandler)
        self.accept('squirrel-into-diamond', self.diamondEventHandler)
        self.accept('squirrel-into-door', self.doorEventHandler)
        
        self.accept('ratMain-into-squirrel', self.ratCatchHandler)
        
        return None
    def ratCatchHandler(self, colEntry):
        
        self.parent.events.doRatCaught()
        return None
        
    def doorEventHandler(self, colEntry):
        print "Door"
        self.parent.events.doExitDoorEvent()
        return None
    
    def ventConvoHandler(self, colEntry):
        name = colEntry.getIntoNode().getName()
        self.parent.events.doVentConvoEvent(name)
        return None
    
    def ventExitHandler(self, colEntry):
        self.parent.events.doVentExit()
        
        return None
    
    
    def diamondEventHandler(self, colEntry):
        self.parent.events.doDiamondEvent()
        
        return None
    
    
    def roofEventHandler(self, colEntry):
        self.parent.events.doRoofExitEvent()
        return None
    #handle the wall collisions for the main character
    def wallCollideHandler(self, colEntry):
        disp = (colEntry.getSurfacePoint(render) -  colEntry.getInteriorPoint(render))
        if(colEntry.getFromNode().getName() == "squirrel"):
            newPos = self.parent.s.squirrel.getPos() + disp
            self.parent.s.squirrel.setPos(newPos)
            
        if (colEntry.getFromNode().getName() == "ratMain"):
            colNp = colEntry.getFromNodePath()
            fromTag = int( colNp.getTag("id") )
            newPos = self.parent.enemyMan.array[fromTag].actor.getPos() + disp
            self.parent.enemyMan.array[fromTag].actor.setPos(newPos)
        return None
   
    #when collided with the wall, turn is wall in vision to true
    def enemyWallVision(self, entry):
        disp = (entry.getSurfacePoint(render) -  entry.getInteriorPoint(render))
        colNp = entry.getFromNodePath()
        fromTag = int( colNp.getTag("id") )
        wallVec = (entry.getSurfacePoint(render) - self.parent.enemyMan.array[fromTag].actor.getPos())
        wall_dist = self.parent.enemyMan.array[fromTag].getWallDist(wallVec)
        #print "in again"          
        return None
    
    #when no longer collided with the wall, turn is wall in vision to false
    def enemyWallVisionOut(self, entry):
        colNp = entry.getFromNodePath()
        fromTag = int( colNp.getTag("id") )
        self.parent.enemyMan.array[fromTag].isWallInVision = False
        #print "out"
        
        return None 
      
    #handle the ground collisions for the main character    
    def groundCollideHandler(self, colEntry):
        if(colEntry.getFromNode().getName() == "squirrel"):
            newZ = colEntry.getSurfacePoint(render).getZ()
            self.parent.s.squirrel.setZ(newZ+0.075)
            

        if (colEntry.getFromNode().getName() == "ratMain"):
            colNp = colEntry.getFromNodePath()
            fromTag = int( colNp.getTag("id") )
            newZ = colEntry.getSurfacePoint(render).getZ()
            self.parent.enemyMan.array[fromTag].actor.setZ(newZ+0.075)
           
        return None
    
    #add solid as a from object to the world's traverser    
    def addCollider( self, cSolid ):
        self.traverser.addCollider( cSolid, self.queue)
        return None
   
    #add solid as a from object to the world's traverser    
    def addColliderEvent( self, cSolid ):
        self.traverser.addCollider( cSolid, self.colHandler)
        return None
            
    #remove solid from world's traverser
    def removeCollider( self, cSolid ):
        self.traverser.removeCollider( cSolid )
        return None
    
