#Taken from SpaceFlight sample on Panda3d website.
# Author: Travis Hawk

from pandac.PandaModules import *
from direct.actor.Actor import Actor
from direct.showbase.DirectObject import DirectObject
from direct.gui.OnscreenText      import OnscreenText
from direct.gui.DirectGui         import *
from direct.gui.DirectButton      import DirectButton





class gameMenu( DirectObject ):
    def __init__( self ):
        # setup background
        self.setupBackground()
        # setup camera
        self.setupCamera( )
        # load splash screen
        self.loadSplashScreen( )
        # load buttons
        self.loadButtons( )
        # display onscreen text
        self.displayOST( )
        # accept key events
        self.setupKeyControls( )
        # setup picker
        self.setupPicker( )
        # play intro theme music
        #self.playIntroMusic( )
    def setupBackground( self ):
        #  set background color
        base.setBackgroundColor( 0.0, 0.0, 0.0 )
    def setupCamera( self ):
        # disable auto controls
        base.disableMouse()
        # set position, heading, pitch, and roll
        camera.setPosHpr( Vec3( 0.0, -22.0, 0.0), Vec3( 0.0, 0.0, 0 ) )
    def loadSplashScreen( self ):
        self.splashScreen = loader.loadModel( '../models/splashScreen.egg' )
        self.splashScreen.reparentTo( render )
        self.splashScreen.setPosHpr( Vec3( 0, 0, 0 ), Vec3( 0, 0, 0 ) )
    #self.exitButton.setScale( 5 )
    def loadButtons( self ):
        # load play button
        self.playButton = loader.loadModel( '../models/playButton.egg' )
        self.playButton.reparentTo( render )
        self.playButton.setPosHpr( Vec3( -4, -4, -3 ), Vec3( 0, 0, 0 ) )
        self.playButton.setScale( 0.3 )
        # load exit button
        self.exitButton = loader.loadModel( '../models/howToButton.egg' )
        self.exitButton.reparentTo( render )
        self.exitButton.setPosHpr( Vec3( -4, -4, -3.8 ), Vec3( 0, 0, 0 ) )
        self.exitButton.setScale( 0.3 )
    def displayOST( self ):
        # display copright information
        self.copyRight = OnscreenText( text = 'Team 2 - Hey Darrell', fg = ( 1.0, 1.0, 1.0, 1.0), pos = ( 1, -.98 ), scale = 0.05 )
        # display panda version text
        #self.pandaVersion = OnscreenText( text = 'Panda Version 1.4.0', fg = ( 1.0, 1.0, 1.0, 1.0), pos = ( -1.18, -.98 ), scale = 0.04 )
    def setupKeyControls( self ):
        # accept the esc key to exit application
        #self.accept( 'escape', sys.exit )
        self.accept( 'mouse1', self.mouseEventHandler )
    def setupPicker( self ):
        # create collision traverser
        self.picker = CollisionTraverser( )
        # create collision handler
        self.picHandEvent = CollisionHandlerEvent( )
        self.picHandEvent.addInPattern( 'into-%in' )
        # create a collision ray
        self.pickerRay = CollisionRay( ) # create collision ray
        # create collision node
        self.pickerNode = CollisionNode( 'mouseRay' ) # create collision node
        # add picker ray to the picker node 
        self.solidIndex = self.pickerNode.addSolid( self.pickerRay ) # add the collision ray to the collision node
        # set bit mask to one 
        self.pickerNode.setIntoCollideMask( BitMask32.bit( 1 ) ) # set bit mask
        # attach new collision node to camera node
        self.pickerNP = camera.attachNewNode( self.pickerNode ) # attach collision node to camera
        # make the traverser know about the picker node and its event handler
        self.picker.addCollider( self.pickerNP, self.picHandEvent ) # add the colision node path and collision handler event
        #self.picker.showCollisions( render ) # render or draw the collisions
        #self.pickerNP.show( ) # render picker ray
        # create col node
        self.playButColPlane = CollisionNode( 'playButColPlane' )
        self.exitButColPlane = CollisionNode( 'exitButColPlane' )
        # add solid to col node plane
        self.playButColPlane.addSolid( CollisionPolygon( Point3( -5, 0, 1.3 ), Point3( 5, 0, 1.3 ), Point3( 5, 0, -1.3 ), Point3( -5, 0, -1.3 ) ) )
        self.exitButColPlane.addSolid( CollisionPolygon( Point3( -5, 0, 1.3 ), Point3( 5, 0, 1.3 ), Point3( 5, 0, -1.3 ), Point3( -5, 0, -1.3 ) ) )
        # attach new node to the render
        self.playButColPlanePath = self.playButton.attachNewNode( self.playButColPlane )
        self.exitButColPlanePath = self.exitButton.attachNewNode( self.exitButColPlane )
        # set heading for collision plane
        self.playButColPlanePath.setH( 180 )
        self.exitButColPlanePath.setH( 180 )
        #self.exitButColPlanePath.setH( 180 )
        # show the node or render it on screen
        #self.playButColPlanePath.show( )
        #self.exitButColPlanePath.show( )
        # set bit mask to one
        # as I understand it, this makes all col nodes with bit mask one
        # create collisions while ignoring others of other masks
        self.playButColPlanePath.node( ).setIntoCollideMask( BitMask32.bit( 1 ) )
        self.exitButColPlanePath.node( ).setIntoCollideMask( BitMask32.bit( 1 ) )
        # listen for collision events
        self.accept( 'into-playButColPlane', self.mouseIntersect )
        self.accept( 'into-exitButColPlane', self.mouseIntersect )
    def mouseEventHandler( self ):
        if base.mouseWatcherNode.hasMouse( ): # if we have a mouse
            mpos = base.mouseWatcherNode.getMouse( ) # get the path to the mouse 
            # shoot ray from camera
            # based on X & Y coordinate of mouse
            self.pickerRay.setFromLens( base.camNode, mpos.getX( ), mpos.getY( ) )
            self.picker.traverse( render )
    def mouseIntersect( self, entry ):
        # get the node path to the into node
        nodePath = entry.getIntoNodePath( )
        # if the node path is equal to the play button collision plane
        if str( nodePath ) == 'render/playButton.egg/playButColPlane':
            applicationFSM.request( 'GameLoop' ) # go into the game loop
        # similar
        elif str( nodePath ) == 'render/howToButton.egg/exitButColPlane':
            sys.exit( ) # exit the game
        else:
            return # do nothing
    #def playIntroMusic( self ):
        # load sound
     #   self.introMusic = loader.loadMusic( 'assets/music/intro.wav' )
        # loop
      #  self.introMusic.setLoop( True )
        # play
       # self.introMusic.play( )
    def cleanup( self ):
        # remove nodes
        self.splashScreen.removeNode( )
        self.exitButton.removeNode( )
        self.playButton.removeNode( )
        self.copyRight.removeNode( )
        self.pickerNode.removeSolid( self.solidIndex )
        self.picker.removeCollider( self.pickerNP )
        # kill the music
        self.introMusic.stop( )