# Author: Darrell Christmas
# Last Updated: 10/01/08
# Key manager to handle keyboard and mouse input
from direct.showbase.DirectObject import DirectObject
from direct.task import Task
import sys

class KeyManager(DirectObject):
    def __init__(self, parent):
        base.disableMouse() 
        self.keyMap = {"left":0, "right":0, "forward":0, "backward":0, "sneak":0, "throw":0, "godview":0}
        self.parent = parent
        self.prevtime = 0
        self.prevtimeShift = 0
        self.isMoving = False
        self.godView = False
        self.shiftPressed = False
        #key acceptors
        self.accept("escape", sys.exit)
        self.accept("a", self.setKey, ["left",1])
        self.accept("s", self.setKey, ["backward",1])
        self.accept("w", self.setKey, ["forward",1])
        self.accept("d", self.setKey, ["right",1])


        self.accept("a-up", self.setKey, ["left",0])
        self.accept("s-up", self.setKey, ["backward",0])
        self.accept("w-up", self.setKey, ["forward",0])
        self.accept("d-up", self.setKey, ["right",0])
        #self.accept("k-up", self.setKey, ["throw", 0])
        
        self.accept ("0", taskMgr.popupControls)
        self.accept("l", self.toggleSneak)
        self.accept("f", parent.createFog)
        
        self.accept("space", self.toggleGodMode)
        #cheat buttons
        self.accept("1", self.stage1)
        self.accept("2", self.stage2)
        self.accept("3", self.stage3)
        self.accept("4", self.stage4)
        taskMgr.add(self.move,"moveTask")
        return None
    
    
    def stage1(self):
        self.parent.s.squirrel.setPos(0,-10,64.5)
        self.parent.s.squirrel.setH(275)
        self.parent.events.loadRoofExitEventTrigger()
        return None
    
    def stage2(self):
        self.parent.s.squirrel.setPos(-20,-23, 45)
        self.parent.s.squirrel.setH(90)
        self.parent.enemyMan.loadRatslvl2()
        self.parent.events.loadTunnelEvents()
        
        return None
    def stage3(self):
        self.parent.s.squirrel.setPos(-96,-2,18)
        self.parent.s.squirrel.setH(0)
        base.camera.setPos(0,35,12)
        self.parent.enemyMan.loadRatslvl3()
        self.parent.events.loadDiamondEvent()
        return None
    def stage4(self):
        self.parent.s.squirrel.setPos(-8,-53,18)
        self.parent.s.squirrel.setH(90)
        self.parent.enemyMan.loadRatslvl3()
        self.parent.events.loadDiamondEvent()
        self.parent.events.doDiamondEvent()
        return None
    
    #set key function
    def setKey(self, key, value):
        self.keyMap[key] = value
        return None
   
    #toggle god mode camera debug mode
    def toggleGodMode(self):
        base.oobe()
        return None
    def toggleSneak(self):
        if(self.shiftPressed == False):
            self.shiftPressed = True
        else:
            self.shiftPressed = False
        #print self.shiftPressed
        return None
    #function to move the main character
    def move(self, task):
        elapsed = task.time - self.prevtime
       
        
        if (self.keyMap["left"]!=0):
            self.parent.s.turnLeft(elapsed, self.shiftPressed)
        if (self.keyMap["right"]!=0):
            self.parent.s.turnRight(elapsed, self.shiftPressed)
        if (self.keyMap["forward"]!=0):
            self.parent.s.moveForward(elapsed, self.shiftPressed)
        if (self.keyMap["backward"]!=0):
            self.parent.s.moveBackward(elapsed, self.shiftPressed)
       

            
                   
        self.prevtime = task.time
        
        if (((self.keyMap["forward"]!=0) or (self.keyMap["left"]!=0) or (self.keyMap["right"]!=0) or (self.keyMap["backward"]!=0 ))and self.shiftPressed == False ):
            #print "regular"
            if self.parent.s.isMoving is False:
                self.parent.s.squirrel.loop("run")
                self.parent.s.isMoving = True
        elif (((self.keyMap["forward"]!=0) or (self.keyMap["left"]!=0) or (self.keyMap["right"]!=0) ) and self.shiftPressed == True ):
            #print "sneak"
            if self.parent.s.isMoving is False:
                self.parent.s.squirrel.loop("sneak")
                self.parent.s.isMoving = True
        elif ( (self.keyMap["backward"]!=0) and self.shiftPressed == True):
            if self.parent.s.isMoving is False:
                self.parent.s.squirrel.loop("sneakBackwards")
                self.parent.s.isMoving = True
        else:
            if self.parent.s.isMoving:
                self.parent.s.squirrel.stop()
                self.parent.s.squirrel.pose("idle",1)
                self.parent.s.isMoving = False
                taskMgr.doMethodLater(5, self.parent.s.idleAnim, 'Idle Animation')
        return Task.cont  


