# Author: Darrell Christmas
# Last Updated: 10/01/08

#from pandac.PandaModules import loadPrcFileData
#loadPrcFileData("", "want-directtools #t")
#loadPrcFileData("", "want-tk #t")

import direct.directbase.DirectStart
from pandac.PandaModules import *
from direct.interval.IntervalGlobal import *
from direct.showbase.DirectObject import DirectObject
from Character.Squirrel import Squirrel
from direct.gui.OnscreenText import OnscreenText
from Character.Rat import Rat
from SoundManager.SoundManager import SoundManager
from GameMenu.GameMenu import gameMenu
from CollisionManager.CollisionManager import CollisionManager
from EnemyManager.EnemyManager import EnemyManager
from KeyManager.KeyManager import KeyManager
from Events.Events import Events
from Screens.Screens import Screens
import random, sys, os, math


class World(DirectObject):

  def __init__(self):
      
      
      self.screens = Screens(self)
      self.screens.introMovie()
 
     
      return None
  
  def loadGame(self):
      
      
      self.colMan = CollisionManager(self)
      #load up players and npcs
      self.s = Squirrel(self)                
      #load up the managers
      self.keyMan = KeyManager(self)
      self.soundMan = SoundManager()
      #self.gameMenu = gameMenu()
      self.events = Events(self)
      self.enemyMan = EnemyManager(self)
      self.loadEnvironment()
      
      
      return None
  #loads the environment and sets up the collision for the building
  def loadEnvironment(self):
      self.fogOn = False
      self.title = self.addTitle("Dick Squirle` Private Investigator")
      self.inst1 = self.addInstructions(0.95,"[ESC]: Quit")
      self.inst2 = self.addInstructions(0.90,"[WASD]: Move Character")
      self.inst3 = self.addInstructions(0.85,"[L]: Sneak")
      self.inst4 = self.addInstructions(0.80,"[F]: Toggle Fog")
      self.inst5 = self.addInstructions(0.75,"[SPACE]: Toggle God Mode")
      
      base.setBackgroundColor(0, 0, 0)    #Set the background to black
      self.building = loader.loadModel("../models/building/building")
      self.building.reparentTo(render)
      self.building.setScale(5)
      self.building.setPos(0,0,0)
      self.building.setHpr(0,0,0)
      #self.introMovie()wll
      #set up collisions
      self.walls = self.building.find("**/wall_collide")
      #self.walls.show()
      self.floorBottom = self.building.find("**/floor_collide")
      #self.floorBottom.show()
      self.boxes = self.building.find("**/box_collide")
      #self.boxes.show()
      #load up the other environment elements
      self.loadSky()
      self.createLights()
      self.createFog()
      self.loadDiamond()
     
      Sequence(Wait(5),Func(self.soundMan.playRoofConversation)).start()

      
      
      self.events.loadRoofExitEventTrigger()
      return None
  

  def addInstructions(self, pos, msg):
        return OnscreenText(text=msg, style=1, fg=(1,1, 1,1), mayChange=1,
                        pos=(-1.3, pos), align=TextNode.ALeft, scale = .05, shadow=(0,0,0,0), shadowOffset=(0.1,0.1))

  def addTitle(self, text):
        return OnscreenText(text=text, style=1, fg=(1,1,1,1),
                    pos=(1.3,-0.95), align=TextNode.ARight, scale = .08)

  
  #loads the skysphere
  def loadSky(self):
      self.sky = loader.loadModel("../models/sky/sky_sphere")
      self.sky.reparentTo(render)
      self.sky.setScale(5)
      return None
  #dims the ambientlight for the outer world
  #TODO: need to make a function to increase the light when going to the interior
  def createLights(self):
      ambientLight = AmbientLight('alight') 
      ambientLight.setColor(Vec4(0.3, 0.3, 0.3, 1)) 
      alnp = render.attachNewNode(ambientLight) 
      #self.s.setLight(alnp) 
      self.building.setLight(alnp)
      return None
 
  def movieCheck(self, task):
      elapsed = task.time - self.prevtime
      
      if task.isPlaying() == True:
          self.prevtime = task.time
          return Task.cont
      else:
          return Task.done 
  
   
  
  #create the fog that engulfs the outside area
  def createFog(self):
    if(self.fogOn==False):
          colour = (0.5,0.5,0.5)
          expfog = Fog("Scene-wide exponential Fog object")
          expfog.setColor(*colour)
          expfog.setExpDensity(0.01)
          render.setFog(expfog)
          self.fogOn=True
    else:
          render.clearFog()
          self.fogOn=False
    return None
  #disables the fog off the screen

  
  def loadDiamond(self):
      self.diamond = loader.loadModel("../models/diamond/diamondnut")
      self.diamond.reparentTo(render)
      self.diamond.setPos(-4,-58,18)
      self.diamond.setScale(2)
      
      
      #add collider to traverser queue
      #self.colMan.addColliderEvent(self.cSolid)    
      return None
  
  
  
  
  
w = World()
run()


